import json
import cherrypy
import core


class ResponseEntity(object):
    class JsonEncoder(json.JSONEncoder):
        def default(self, o):
            return o.__dict__
    _json_encoder = JsonEncoder()

    def __init__(self, status='ok', data='', error=''):
        self.status = status
        if data != '':
            self.data = data
        if error != '':
            self.error = error

    def tojson(self):
        return ResponseEntity._json_encoder.encode(self)


class FormConstraint(object):
    def __init__(self, name, required='true', type='string', min=0, max=0xffffffff, default=None, nested=None):
        self.name = name
        self.required = required
        self.type = type
        self.min = min
        self.max = max
        self.default = default
        self.nested = nested


class FormBinder(object):
    def __init__(self, form, form_template):
        self.errors = []
        self._form_template = form_template
        self._bind_form(form)

    def has_errors(self):
        return len(self.errors) > 0

    def _bind_form(self, form):
        constraints = [getattr(self._form_template, constraint)
                       for constraint in self._form_template.__dict__
                       if isinstance(getattr(self._form_template, constraint), FormConstraint)]
        fields = {}
        for constraint in constraints:
            value = form.get(constraint.name, None)

            if constraint.nested is not None:
                if value is None and constraint.required:
                    self.errors.append({'field': constraint.name, 'error': 'MISSING_FIELD'})
                    continue
                elif value is not None:
                    another_binder = FormBinder(value, constraint.nested)
                    if another_binder.has_errors():
                        self.errors.extend(another_binder.errors)
                    value = another_binder.form
            else:
                if value is None and constraint.required:
                    self.errors.append({'field': constraint.name, 'error': 'MISSING_FIELD'})
                    continue
                elif value is None and not constraint.required:
                    value = constraint.default

                if value is not None:
                    if constraint.type == 'int':
                        try:
                            int(value)
                            if value > constraint.max:
                                self.errors.append({'field': constraint.name, 'error': 'MAX'})
                            if value < constraint.min:
                                self.errors.append({'field': constraint.name, 'error': 'MIN'})
                        except ValueError:
                            self.errors.append({'field': constraint.name, 'error': 'BAD_TYPE'})
                    elif constraint.type == 'float':
                        try:
                            float(value)
                            if value > constraint.max:
                                self.errors.append({'field': constraint.name, 'error': 'MAX'})
                            if value < constraint.min:
                                self.errors.append({'field': constraint.name, 'error': 'MIN'})
                        except ValueError:
                            self.errors.append({'field': constraint.name, 'error': 'BAD_TYPE'})
                    elif constraint.type == 'string':
                        try:
                            str(value)
                            if len(value) > constraint.max:
                                self.errors.append({'field': constraint.name, 'error': 'MAX_LENGTH'})
                            if len(value) < constraint.min:
                                self.errors.append({'field': constraint.name, 'error': 'MIN_LENGTH'})
                        except ValueError:
                            self.errors.append({'field': constraint.name, 'error': 'BAD_TYPE'})
            fields[constraint.name] = value
        self.form = core.Context(fields)


def exception_handler():
    cherrypy.response.status = 500
    cherrypy.response.body = ResponseEntity(status='error', error='Internal error').tojson()


class BaseController(object):
    _cp_config = {'request.error_response': exception_handler,
                  'show_tracebacks': False,
                  'show_mismatched_params': False,
                  'throw_errors': False}

    def __init__(self):
        self.context = None

    def initialize(self):
        pass

    @cherrypy.expose
    def default(self, *vpath, **params):
        def find_handling_method():
            return getattr(self, cherrypy.request.method, None)

        def handle_form():
            if cherrypy.request.method in ('POST', 'PUT'):
                try:
                    return json.load(cherrypy.request.body)
                except ValueError:
                    raise ValueError('Cannot parse request body')
            return None

        def setup_response_headers():
            cherrypy.response.headers['Content-Type'] = 'application/json'

        def dispatch_event(method, form):
            if not method:
                cherrypy.response.status = 404
                return ResponseEntity(status='error', error="Not found")
            if form is not None:
                params['form'] = form
            return method(*vpath, **params)

        try:
            setup_response_headers()
            form = handle_form()
            method = find_handling_method()
            result = dispatch_event(method, form)
            if result.status == 'error' and cherrypy.response.status != 404:
                cherrypy.response.status = 400
            return result.tojson()
        except ValueError:
            cherrypy.response.status = 400
            return ResponseEntity(status='error', error='Bad Request').tojson()



import requests
import json
import cherrypy
import threading
import time


class Module(object):
    name = 'eureka'

    def setup_engine(self, loaded_properties, args):
        app_name = loaded_properties.get('app_name')
        remote_config_url = loaded_properties.get('remote_config_url')
        public_hostname = loaded_properties.get('public_hostname')

        if app_name is None or not app_name:
            cherrypy.log.error('Missing app_name! Skipping registration in eureka cluster')
            return
        if remote_config_url is None or not remote_config_url:
            cherrypy.log.error('Missing remote_config_url! Skipping registration in eureka cluster')
            return
        if public_hostname is None or not public_hostname:
            cherrypy.log.error('Missing public_hostname! Skipping registration in eureka cluster')
            return

        if not remote_config_url.endswith('/'):
            remote_config_url += '/'
        remote_config_url += 'eureka'

        _register_instance(remote_config_url, app_name, public_hostname)
        _register_heartbeat_service(remote_config_url, app_name, public_hostname)

        return remote_config_url

    def register_custom_beans(self, remote_config_url, args):
        return {'_eureka_url': remote_config_url}

    def after_setup(self, context, args):
        pass


def list_instances(context, service_name):
    response = requests.get(context._eureka_url + '/apps/' + service_name, headers={'accept': 'application/json'})
    if response.status_code < 200 or response.status_code >= 300:
        raise Exception('Cannot retrieve instances of service ' + service_name)

    instances_list = response.json()['application']['instance']
    return [instance['hostName'] + ':' + str(instance['port']['$']) for instance in instances_list]


def _register_instance(eureka_url, app_name, hostname):
    data_center_info = {
        "@class":"com.netflix.appinfo.InstanceInfo$DefaultDataCenterInfo",
        "name": "MyOwn"
    }

    instance_data = {
        'instance': {
            'hostName': hostname,
            'ipAddr': hostname,
            'app': app_name,
            'status': 'UP',
            'port': {
                "$": 8080,
                "@enabled": 'true'
            },
            'dataCenterInfo': data_center_info
        }
    }

    try:
        response = requests.post(eureka_url + '/apps/' + app_name, json.dumps(instance_data),
                                 headers={'Content-Type': 'application/json'})
        if response.status_code < 200 or response.status_code >= 300:
            cherrypy.log.error('Cannot register instance on eureka server! Status code ' + str(response.status_code))
    except requests.exceptions.ConnectionError:
        cherrypy.log.error('Cannot connect to eureka server!')


def _send_heartbeat(eureka_url, app_name, hostname):
    response = requests.put(eureka_url + '/apps/' + app_name + '/' + hostname,
                            headers={'Content-Type': 'application/json'})
    if response.status_code < 200 or response.status_code >= 300:
        cherrypy.log.error('Sending heartbeat to eureka cluster failed! Status code ' + str(response.status_code))


def _register_heartbeat_service(remote_config_url, app_name, public_hostname):
    def heartbeat_sending_thread():
        while True:
            time.sleep(10)
            _send_heartbeat(remote_config_url, app_name, public_hostname)

    thread = threading.Thread(target=heartbeat_sending_thread)
    thread.daemon = True
    thread.start()



import redis
import cherrypy

DEFAULT_REDIS_PORT = 6379


class Module(object):
    name = 'redisdb'

    def setup_engine(self, loaded_properties, args):
        redis_host = loaded_properties.get('redis_host')
        redis_port = loaded_properties.get('redis_port')

        if redis_host is None or not redis_host:
            cherrypy.log.error('Missing redis_host!')
            return
        if redis_port is None or not redis_port:
            cherrypy.log.error('Missing redis_port! Setting default value ' + str(DEFAULT_REDIS_PORT))
            redis_port = DEFAULT_REDIS_PORT

        return redis.ConnectionPool(host=redis_host, port=int(redis_port), db=0)

    def register_custom_beans(self, connection_pool, args):
        return {'_redis_pool': connection_pool}

    def after_setup(self, context, args):
        pass


def get_connection(context):
    return redis.Redis(connection_pool=context._redis_pool)



import sqlalchemy
import sqlalchemy.orm
import sqlalchemy.ext.declarative
import cherrypy
import contextlib


Table = sqlalchemy.ext.declarative.declarative_base()


class Module(object):
    name = 'db'

    def setup_engine(self, loaded_properties, args):
        database_url = loaded_properties.get('database_url')

        if database_url is None or not database_url:
            cherrypy.log.error('No database url found in properties. Skipping ORM engine creation.')
            return None

        # mysql+pymysql://{username}:{password}@url
        return sqlalchemy.create_engine(database_url)

    def register_custom_beans(self, db_engine, args):
        return {'_session_maker': sqlalchemy.orm.sessionmaker(bind=db_engine)} if db_engine is not None else {}

    def after_setup(self, context, args):
        pass


@contextlib.contextmanager
def transaction(context):
    session = None
    try:
        session = context._session_maker()
        yield session
        session.commit()
    except Exception as e:
        if session is not None:
            session.rollback()
        raise e
    finally:
        if session is not None:
            session.close()


class GenericRepository(object):
    def all(self, transaction):
        return transaction.query(self.__basetable__).all()

    def one(self, transaction, id):
        return transaction.query(self.__basetable__).filter(self.__basetable__.id == id).one()

    def add(self, transaction, entity):
        transaction.add(entity)

    def delete(self, transaction, entity):
        transaction.delete(entity)


class DataAccessException(Exception):
    pass



class Module(object):
    name = '_beans'

    def setup_engine(self, loaded_properties, args):
        return args.get('beans', [])

    def register_custom_beans(self, beans, args):
        return {bean.path: bean.bean for bean in beans}

    def after_setup(self, context, args):
        for bean in args.get('beans', []):
            setattr(bean.bean, 'context', context)
            if hasattr(bean.bean, 'initialize'):
                bean.bean.initialize()



import ConfigParser
import collections
import logging
from logging import handlers
import beans
import cherrypy
import requests


DEFAULT_HOST = '127.0.0.1'
DEFAULT_PORT = 8000
DEFAULT_APPLICATION_LOG = 'log/application.log'
DEFAULT_ACCESS_LOG = 'log/access.log'
DEFAULT_MAX_LOGS = 10
DEFAULT_MAX_LOG_SIZE = 10
MB_SIZE = 1000000
Mapping = collections.namedtuple('Mapping', ['bean', 'path'])


class Context(object):
    def __init__(self, entries):
        self.__dict__.update(entries)


def _setup_logging(load_properties):
    log = cherrypy.log
    log.error_file = ""
    log.access_file = ""

    maxBytes = getattr(log, "rot_maxBytes", load_properties.get('logs_max_size', DEFAULT_MAX_LOG_SIZE) * MB_SIZE)
    backupCount = getattr(log, "rot_backupCount", load_properties.get('logs_max_files', DEFAULT_MAX_LOGS))

    fname = getattr(log, "rot_error_file", load_properties.get('logs_application_file', DEFAULT_APPLICATION_LOG))
    error_log = handlers.RotatingFileHandler(fname, 'a', maxBytes, backupCount)
    error_log.setLevel(logging.INFO)
    error_log.setFormatter(cherrypy._cplogging.logfmt)
    log.error_log.addHandler(error_log)

    fname = getattr(log, "rot_access_file", load_properties.get('logs_access_file', DEFAULT_ACCESS_LOG))
    h = handlers.RotatingFileHandler(fname, 'a', maxBytes, backupCount)
    h.setLevel(logging.INFO)
    h.setFormatter(cherrypy._cplogging.logfmt)
    log.access_log.handlers = []
    log.access_log.addHandler(h)


def _update_config(load_properties):
    cherrypy.config.update({'server.socket_port': load_properties.get('server_port', DEFAULT_PORT),
                            'server.socket_host': load_properties.get('server_host', DEFAULT_HOST)})


def _load_properties(file):
    parser = ConfigParser.RawConfigParser()
    try:
        parser.readfp(open(file, 'r'))
    except IOError:
        cherrypy.log.error('Cannot open properties file {0}'.format(file))
        raise IOError('Cannot open properties file {0}'.format(file))
    return {key: value for (key, value) in parser.items('Properties')}


def _load_remote_configuration(properties):
    remote_config_url = properties.get('remote_config_url')
    app_name = properties.get('app_name')

    if remote_config_url is None or not remote_config_url:
        cherrypy.log.error('Remote config URL not present. Skipping.')
        return properties
    if app_name is None or not app_name:
        cherrypy.log.error('Remote config URL is present but applciation name was not specified!')
        return properties

    if not remote_config_url.endswith('/'):
        remote_config_url += '/'

    remote_properties = {}
    try:
        config_server_response = requests.get(remote_config_url + app_name + '/default')
        if config_server_response.status_code != 200:
            cherrypy.log.error('Error loading remote properties! Status code ' + str(config_server_response.status_code))
            return properties

        sources = [source['source'] for source in config_server_response.json()['propertySources']]
        for source in sources:
            remote_properties.update(source)
    except requests.exceptions.ConnectionError:
        cherrypy.log.error('Error while connecting to remote config server!')
        return properties

    new_properties = properties.copy()
    new_properties.update(remote_properties)
    return new_properties


def _create_context(loaded_properties, modules, kwargs):
    beans = {}
    for module in modules:
        engine = module.setup_engine(loaded_properties, kwargs)
        var_name = module.name + '_engine'
        beans[var_name] = engine
        beans.update(module.register_custom_beans(engine, kwargs))

    return Context(beans)


def _register_controllers(context, controllers_mappings):
    for m in controllers_mappings:
        m.bean.context = context
        m.bean.initialize()
        cherrypy.tree.mount(m.bean, m.path)


def _after_setup(context, modules, kwargs):
    for module in modules:
        module.after_setup(context, kwargs)


def _shutdown_modules(context, modules):
    for module in modules:
        module.shutdown(context)


def init_context(properties,
                 controllers_mappings,
                 modules=(),
                 **kwargs):
    modules = (beans.Module(),) + modules

    loaded_properties = _load_properties(properties)
    loaded_properties = _load_remote_configuration(loaded_properties)
    _update_config(loaded_properties)
    _setup_logging(loaded_properties)
    context = _create_context(loaded_properties, modules, kwargs)

    _register_controllers(context, controllers_mappings)
    _after_setup(context, modules, kwargs)

    return cherrypy.tree


def start_standalone_application():
    cherrypy.engine.start()
    cherrypy.engine.block()



import cherrypy
import pika
import threading


DEFAULT_AMQP_PORT = 5672


class Module(object):
    name = 'amqp'

    def setup_engine(self, loaded_properties, args):
        def parse_address():
            broker_address = loaded_properties['broker_address']
            address_parts = broker_address.split(':')

            if len(address_parts) == 1:
                broker_host = broker_address
                broker_port = DEFAULT_AMQP_PORT
            elif len(address_parts) == 2:
                broker_host = address_parts[0]
                broker_port = int(address_parts[1])
            else:
                cherrypy.log.error('Invalid broker address!')
                return None, None
            return broker_host, broker_port

        broker_username = loaded_properties['broker_username']
        broker_password = loaded_properties['broker_password']
        broker_host, broker_port = parse_address()

        credentials = pika.PlainCredentials(broker_username, broker_password)
        return pika.ConnectionParameters(broker_host, broker_port, '/', credentials)

    def register_custom_beans(self, broker_engine, args):
        return {'_amqp_connection_cache':{}}

    def after_setup(self, context, args):
        listeners_mappings = args.get('listeners_mappings', [])
        for m in listeners_mappings:
            m.bean.context = context
            m.bean.initialize()
            _register_listener(context, m.path, m.bean.on_message)


class BaseListener(object):
    def __init__(self):
        self.context = None

    def initialize(self):
        pass

    def on_message(self, channel, method, properties, body):
        pass


def get_connection(context):
    if threading.currentThread().name in context._amqp_connection_cache:
        return context._amqp_connection_cache[threading.currentThread().name]
    else:
        new_connection = pika.BlockingConnection(context.amqp_engine)
        context._amqp_connection_cache[threading.currentThread().name] = new_connection
        return new_connection


def send_message(context, routing_key, message, durable=True, exchange=''):
    sending_connection = get_connection(context).channel()
    sending_connection.queue_declare(queue=routing_key, durable=durable)
    sending_connection.basic_publish(exchange=exchange,
                                     routing_key=routing_key,
                                     body=message,
                                     properties=pika.BasicProperties(
                                         delivery_mode=2,
                                     ))


def _register_listener(context, routing_key, callback):
    thread_local_channel = get_connection(context).channel()

    def receive_action(channel, method, properties, body):
        try:
            callback(channel, method, properties, body)
            channel.basic_ack(delivery_tag=method.delivery_tag)
        except Exception as e:
            cherrypy.log.error('Error receiving message from queue {0}, exception: {1}'.format(routing_key, e))

    def listener_thread():
        thread_local_channel.queue_declare(queue=routing_key, durable=True)
        thread_local_channel.basic_qos(prefetch_count=1)
        thread_local_channel.basic_consume(receive_action, routing_key)
        thread_local_channel.start_consuming()

    thread = threading.Thread(target=listener_thread)
    thread.daemon = True
    thread.start()
